searchData={"items":[{"type":"module","title":"Mutex","doc":"This is the main module in this application, it implements a mutex as a\nGenServer with a notification system to be able to await lock releases.\n\nSee [`README.md`](https://hexdocs.pm/mutex/readme.html) for how to use.","ref":"Mutex.html"},{"type":"function","title":"Mutex.await/3","doc":"Locks a key if it is available, or waits for the key to be freed before\nattempting again to lock it.\n\nReturns the lock or fails with a timeout.\n\nDue to the notification system, multiple attempts can be made to lock if\nmultiple processes are competing for the key.\n\nSo timeout will be *at least* for the passed amount of milliseconds, but may\nbe slightly longer.\n\nDefault timeout is `5000` milliseconds. If the timeout is reached, the caller\nprocess exists as in `GenServer.call/3`.\nMore information in the [timeouts](https://hexdocs.pm/elixir/GenServer.html#call/3-timeouts)\nsection.","ref":"Mutex.html#await/3"},{"type":"function","title":"Mutex.await_all/2","doc":"Awaits multiple keys at once. Returns once all the keys have been locked,\ntimeout is `:infinity`.\n\nIf two processes are trying to lock `[:user_1, :user_2]` and\n`[:user_2, :user_3]` at the same time, this function ensures that no deadlock\ncan happen and that one process will eventually lock all the keys.\n\nMore information at the end of the\n[deadlocks section](https://hexdocs.pm/mutex/readme.html#avoiding-deadlocks).","ref":"Mutex.html#await_all/2"},{"type":"function","title":"Mutex.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Mutex.html#child_spec/1"},{"type":"function","title":"Mutex.get_meta/1","doc":"Fetch the metadata of the mutex.","ref":"Mutex.html#get_meta/1"},{"type":"function","title":"Mutex.goodbye/1","doc":"Tells the mutex to release *all* the keys owned by the calling process and\nreturns immediately with `:ok`.","ref":"Mutex.html#goodbye/1"},{"type":"function","title":"Mutex.init/1","doc":"","ref":"Mutex.html#init/1"},{"type":"function","title":"Mutex.lock/2","doc":"Attemps to lock a resource on the mutex and returns immediately with the\nresult, which is either a `Mutex.Lock` structure or `{:error, :busy}`.","ref":"Mutex.html#lock/2"},{"type":"function","title":"Mutex.lock!/2","doc":"Attemps to lock a resource on the mutex and returns immediately with the lock\nor raises an exception if the key is already locked.","ref":"Mutex.html#lock!/2"},{"type":"function","title":"Mutex.release/2","doc":"Tells the mutex to free the given lock and immediately returns `:ok` without\nwaiting for the actual release.\nIf the calling process is not the owner of the key(s), the key(s) is/are\n*not* released and an error is logged.","ref":"Mutex.html#release/2"},{"type":"function","title":"Mutex.start/1","doc":"Starts a mutex with no process linking. Given options are passed as options\nfor a `GenServer`, it's a good place to set the name for registering the\nprocess.\n\nSee [`GenServer` options](https://hexdocs.pm/elixir/GenServer.html#t:options/0).\n\nA `:meta` key can also be given in options to set the mutex metadata.","ref":"Mutex.html#start/1"},{"type":"function","title":"Mutex.start_link/1","doc":"Starts a mutex with linking under a supervision tree. Given options are passed\nas options for a `GenServer`, it's a good place to set the name for\nregistering the process.\n\nSee [`GenServer` options](https://hexdocs.pm/elixir/GenServer.html#t:options/0).\n\nA `:meta` key can also be given in options to set the mutex metadata.","ref":"Mutex.html#start_link/1"},{"type":"function","title":"Mutex.under/4","doc":"Awaits a lock for the given key, executes the given fun and releases\nthe lock immediately.\n\nIf an exeption is raised or thrown in the fun, the lock is\nautomatically released.\n\nIf a function of arity 1 is given, it will be passed the lock.\nOtherwise the arity must be 0. You should not manually release the\nlock within the function.","ref":"Mutex.html#under/4"},{"type":"function","title":"Mutex.under_all/3","doc":"Awaits a lock for the given keys, executes the given fun and\nreleases the lock immediately.\n\nIf an exeption is raised or thrown in the fun, the lock is\nautomatically released.\n\nIf a function of arity 1 is given, it will be passed the lock.\nOtherwise the arity must be 0. You should not manually release the\nlock within the function.","ref":"Mutex.html#under_all/3"},{"type":"type","title":"Mutex.key/0","doc":"A key can be any term.","ref":"Mutex.html#t:key/0"},{"type":"type","title":"Mutex.name/0","doc":"The name of a mutex is an atom, registered with `Process.register/2`","ref":"Mutex.html#t:name/0"},{"type":"module","title":"Mutex.Lock","doc":"This module defines a struct containing the key(s) locked with all the\nlocking functions in `Mutex`.","ref":"Mutex.Lock.html"},{"type":"function","title":"Mutex.Lock.get_meta/1","doc":"Returns the metadata associated with the lock. The metadata is given\nto the mutex on initialization:\n\n    Mutex.start_link(name: MyMutex, meta: metadata)","ref":"Mutex.Lock.html#get_meta/1"},{"type":"type","title":"Mutex.Lock.t/0","doc":"The struct containing the key(s) locked during a lock operation. `:type`\nspecifies wether there is one or more keys.","ref":"Mutex.Lock.html#t:t/0"},{"type":"extras","title":"License","doc":"# The MIT License\n\nCopyright (c) 2017, Ludovic Demblans\n\nPermission is hereby granted, free of charge, to any person obtaining a copy of\nthis software and associated documentation files (the \"Software\"), to deal in\nthe Software without restriction, including without limitation the rights to\nuse, copy, modify, merge, publish, distribute, sublicense, and/or sell copies\nof the Software, and to permit persons to whom the Software is furnished to do\nso, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"},{"type":"extras","title":"Overview","doc":"# A simple mutex for Elixir\n\n[![Module Version](https://img.shields.io/hexpm/v/mutex.svg)](https://hex.pm/packages/mutex)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/mutex/)\n[![License](https://img.shields.io/hexpm/l/mutex.svg)](https://github.com/lud/mutex/blob/master/LICENSE.md)\n[![Last Updated](https://img.shields.io/github/last-commit/lud/mutex.svg)](https://github.com/lud/mutex/commits/master)\n\n`Mutex` is a simple mutex module that fits under your supervision tree and allows processes to work on shared ressources one by one. This can be a simple alternative to database transactions. Also, `Mutex` supports multiple keys locking without deadlocks.","ref":"readme.html"},{"type":"extras","title":"Installation - Overview","doc":"This package can be installed by adding `:mutex` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:mutex, \"~> 1.3\"},\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Using Mutex - Overview","doc":"A mutex is handled by a process that you start in your supervision tree with [`child_spec(name)`](https://hexdocs.pm/mutex/Mutex.html#child_spec/1).\n\nOptions can be an atom (used as the `GenServer` name), or a `Keyword` with [`GenServer` options](https://hexdocs.pm/elixir/GenServer.html#t:options/0) and a `:meta` option to set the metadata.\n\n```elixir\nchildren = [\n  {Mutex, name: MyMutex, meta: some_data}\n]\n{:ok, _pid} = Supervisor.start_link(children, strategy: :one_for_one)\n```\n\nLet's see a bad example of concurrent code without database transactions.\n\n```elixir\nresource_id = :some_user_id # unused in example\nupdate_user = fn(worker) ->\n  IO.puts \"[#{worker}] Reading user from database.\"\n  Process.sleep(250)\n  IO.puts \"[#{worker}] Working with user.\"\n  Process.sleep(250)\n  IO.puts \"[#{worker}] Saving user in database.\"\n  Process.sleep(500)\nend\nspawn(fn -> update_user.(\"worker 1\") end)\nspawn(fn -> update_user.(\"worker 2\") end)\nspawn(fn -> update_user.(\"worker 3\") end)\n\n# Results :\n# [worker 1] Reading user from database.\n# [worker 2] Reading user from database.\n# [worker 3] Reading user from database.\n# [worker 1] Working with user.\n# [worker 2] Working with user.\n# [worker 3] Working with user.\n# [worker 2] Saving user in database. # 2 before 1 !\n# [worker 1] Saving user in database.\n# [worker 3] Saving user in database.\n```\n\nSerialized transactions could be a good fit for this specific case, but let's see another simple solution :\n\nWith a simple mutex mechanism, workers will be able to wait until the resource is saved in database before loading it, with the guarantee that any other worker will not be able to touch the resource.\n\nEach worker will wait for the key identifying the resource to be available on the mutex and attempt to lock it. A key can be anything : `:my_resource`, `{User, user_id}`, `\"http://example.com\"` or more complex structures.\n\nWhen the lock is eventually acquired, the worker can work with the resource. Of course, this works only if all your workers use the mutex, with the same key.\n\n```elixir\nresource_id = {User, {:id, 1}}\nupdate_user = fn(worker) ->\n  lock = Mutex.await(MyMutex, resource_id)\n  IO.puts \"[#{worker}] Reading user from database.\"\n  Process.sleep(250)\n  IO.puts \"[#{worker}] Working with user.\"\n  Process.sleep(250)\n  IO.puts \"[#{worker}] Saving user in database.\"\n  Mutex.release(MyMutex, lock)\nend\nspawn(fn -> update_user.(\"worker 4\") end)\nspawn(fn -> update_user.(\"worker 5\") end)\nspawn(fn -> update_user.(\"worker 6\") end)\n\n# [worker 4] Reading user from database.\n# [worker 4] Working with user.\n# [worker 4] Saving user in database.\n# [worker 5] Reading user from database.\n# [worker 5] Working with user.\n# [worker 5] Saving user in database.\n# [worker 6] Reading user from database.\n# [worker 6] Working with user.\n# [worker 6] Saving user in database.\n```","ref":"readme.html#using-mutex"},{"type":"extras","title":"Error Handling - Overview","doc":"Whenever a process that locked a key on the mutex crashes, the mutex\nautomatically unlocks the key so any other process can lock it in its turn.\n\nBut if you catch exceptions you may forget to release the keys and keep\nunnecessary keys locked for a while :\n\n```elixir\n# Do not do this\ntry do\n  lock = Mutex.await(MyMutex, :some_key)\n  throw(:fail)\n  # This will never happen:\n  Mutex.release(MyMutex, lock)\ncatch\n  :throw, :fail -> :ok\nend\n```\n\nWhenever possible, avoid to lock keys in `try`, `if`, `for`, ... blocks.\n\nThe mutex provides a mechanism to automatically handle this situation. Using `Mutex.under/3`, the calling process will wait for the key to be available and lock it. Then the passed `fn` will be executed and if it raises or throws, the lock will automatically be removed. Exceptions and thrown values are reraised and rethrown so you still have to handle them.\n\n```elixir\n# Do this instead\ntry do\n  Mutex.under(MyMutex, :some_key, fn ->\n    throw(:fail)\n  end)\ncatch\n  :throw, :fail -> :ok\nend\n```\n\nA multilock version is also available with `Mutex.under_all/3`.\n\nBoth functions can accept a fun of arity `1` that will be passed the lock.","ref":"readme.html#error-handling"},{"type":"extras","title":"Avoiding Deadlocks - Overview","doc":"A deadlock would occur if the keys were locked one by one with a race condition :\n\n```elixir\n  # Do not do this\n\n  def handle_order(buyer, seller) do\n    lock1 = Mutex.await(MyMutex, buyer)\n    lock2 = Mutex.await(MyMutex, seller)\n    do_some_work_with_users(buyer, seller)\n    Mutex.release(MyMutex, lock1)\n    Mutex.release(MyMutex, lock2)\n  end\n\n  spawn(fn -> handler_order(:user_1, :user_2) end) # Process 1\n  spawn(fn -> handler_order(:user_2, :user_1) end) # Process 2\n```\n\nProcess 1 will first lock `:user_1` and process 2 will lock `:user_2`, and then each process is waiting for the key that is already locked by the other one.\n\n**If any process should have, at any given time, several keys locked, those keys shall have been locked all at once.**\n\nThis simple rule is mandatory and sufficient to be free from deadlocks, and `Mutex.await_all/2` is the simplest way to respect that rule.\n\n```elixir\n# Do this instead\n\ndef handle_order(buyer, seller) do\n  lock = Mutex.await_all(MyMutex, [buyer, seller])\n  do_some_work_with_users(buyer, seller)\n  Mutex.release(MyMutex, lock)\nend\n```\n\nIf you really have to lock keys in a loop, or in mutiple moments, the `Mutex.goodbye/1` function allows to simply release all the keys locked by the calling process in one call.","ref":"readme.html#avoiding-deadlocks"},{"type":"extras","title":"Metadata - Overview","doc":"A mutex can hold metadata that will be assigned to each lock. The metadata is set upon initialization (given as `:meta` in the child spec in your supervisor, or in the options for `Mutex.start` or `Mutex.start_link`).\n\nMetadata can be fetched at anytime with `Mutex.get_meta/1`.\n\nThe metadata is also sent to any client that locks a key or a group of keys:\n\n```elixir\n{:ok, pid} = Mutex.start_link(meta: :some_data)\n{:ok, lock} = Mutex.lock(pid, :some_key)\nlock.meta === :some_data\nMutex.Lock.get_meta(lock) === :some_data\n```\n\nThe lock will also be passed to a fun if its arity is `1` when using `Mutex.under/4` and `Mutex.under_all/3`. The arity of the fun can also be `0`. Releasing the lock within the fun is still useless as it will be automatically released as for 0-arity funs, and could give other processes the ability to lock the keys before the fun execution is complete.","ref":"readme.html#metadata"},{"type":"extras","title":"Copyright and License - Overview","doc":"Copyright (c) 2017, Ludovic Demblans\n\nThis work is free. You can redistribute it and/or modify it under the\nterms of the MIT License. See the [LICENSE.md](./LICENSE.md) file for more details.","ref":"readme.html#copyright-and-license"}],"content_type":"text/markdown"}